@ECHO OFF
CLS

:init
REM Variables used in this script :
REM ERR : error flag - to avoid losing time by interrupting the whole process when an error is detected, and to display an appropriate error message
REM BF : Backup Folder
REM TD : our own Temporary Directory, where the converted files shall be created
REM ResDir : relative path from current directory to the CRF files ; defaulted to "." - we'll change that to "RES" if it seems to be there
REM BFR : relative path to the game's home directory, Back From ResDir : if we set ResDir to "RES", BFR will be changed to ".."
REM We'll need BFR to call commands from within our Temporary ResDir
REM ExeVer : will keep track of the version of Thief.exe we start with ; init at "?" for unknown/undefined
SET ERR=0
SET BF=GTD_Backup
SET TD=GTD.tmp
SET ResDir=.
SET BFR=.
SET ExeVer=?

ECHO  ͻ
ECHO                        GoldToDark : version 2014-08-02B                      
ECHO  ͼ
ECHO        Downgrade script from 'Thief Gold' to 'Thief: the Dark Project', 
ECHO        made by some bloody taffer who goes by the name of Constance.    
ECHO       ͼ
ECHO.
ECHO  ******************************************************************************
ECHO  *       Please ensure you have read the "GoldToDark - Read me.txt" file      *
ECHO  *          or equivalent in your language before running this patch.         *
ECHO  ******************************************************************************
ECHO.
ECHO Press Ctrl C to stop this script, or any other key to start testing files.
PAUSE >NUL
:step1
ECHO.
ECHO STEP 1 - Verifying presence and integrity of patch data...
FOR %%a in (md5sum.exe libiconv2.dll libintl3.dll regex2.dll GTD.md5) DO IF NOT EXIST GTD\%%a SET ERR=1
IF NOT %ERR%==0 GOTO ERREUR
REM Moving down the GTD directory, where all the patcher's data files are
CD GTD
REM Checking the hash for patch data hashlist
ECHO 696C7A95BA864F4D277EBB2545B79991 *GTD.MD5 | .\md5sum -c
IF ERRORLEVEL 1 SET ERR=1
IF NOT %ERR%==0 GOTO ERREUR
REM Checking the patch data according to the hashlist we just checked
.\md5sum -c GTD.md5
IF ERRORLEVEL 1 SET ERR=1
IF NOT %ERR%==0 GOTO ERREUR
REM Moving back up to Thief's main directory...
CD ..

:step2a
ECHO.
ECHO STEP 2-a - Verifying presence and integrity of game data - part 1...
GTD\md5sum -c GTD\TGpart1.md5
IF ERRORLEVEL 1 SET ERR=2
IF NOT %ERR%==0 GOTO ERREUR

:step2b
ECHO.
ECHO STEP 2-b - Verifying presence and integrity of game data - part 2...
REM Here we try to guess where our .CRF files are, according to the existence or not of a SND.CRF within some RES subdirectory
IF EXIST RES\SND.CRF SET ResDir=RES
IF "%ResDir%"=="RES" SET BFR=..
REM Moving further down to the RES subdirectory if it exists - if not we'll still be Thief's main directory.
REM BFR will then come in handy to get back to previous location ^^
CD %ResDir%
%BFR%\GTD\md5sum -c %BFR%\GTD\GOLDCRF.md5
IF ERRORLEVEL 1 SET ERR=2
REM Coming back to Thief's main directory
CD %BFR%
IF NOT %ERR%==0 GOTO ERREUR

:step2c
ECHO.
ECHO STEP 2-c - Verifying if Thief.exe is in a version we know and can handle...
REM Possible values for ExeVer will be :
REM ? = unknown / undefined, if it stays that way, we'll assume it must be some NewDark variant and ignore it
REM 137 = same as on the original CDs and Mastertronic's Complete Collection
REM 137a = same as the one in GOG.com's release as of 2012.02.02 - includes multicore fix but not ddfix
REM 137b = same as the one in Steam's release - also in GOG.com's release as of 2013.01.03 - DDfixed version
ECHO 8DD515DF75176BCD1CAAF65A93F47D81 *Thief.exe | GTD\md5sum -c --status
IF NOT ERRORLEVEL 1 SET ExeVer=137
ECHO 3461838E337F02877DA7E46891EC665C *Thief.exe | GTD\md5sum -c --status
IF NOT ERRORLEVEL 1 SET ExeVer=137a
ECHO 99C41D42D130A33172BDCF5DBA8EE85B *Thief.exe | GTD\md5sum -c --status
IF NOT ERRORLEVEL 1 SET ExeVer=137b
IF %ExeVer%==? ECHO Thief.exe was not recognized or not found, I'll assume it's a NewDark variant.
IF NOT %ExeVer%==? ECHO OK, Thief.exe identified as '%ExeVer%' - for internal reference only

:step3
ECHO.
ECHO All required files seem to be OK.
ECHO.
ECHO Press Ctrl C to abort or any other key to proceed with patching.
PAUSE >NUL
ECHO.
ECHO STEP 3 - Generating new files...
ECHO Creating temporary work directories.
MD %TD%
MD %TD%\MOVIES
IF NOT %ResDir%==. MD %TD%\%ResDir%
REM xdelta patch syntax reminder : xdelta patch patchfile [fromfile [tofile]]
ECHO Generating the CRF files...
CD %ResDir%
REM current directory now should be the Game's original %ResDir%
FOR %%a IN (bitmap books default editor fam intrface mesh motions obj pal snd strings) DO %BFR%\GTD\xdelta patch %BFR%\GTD\RES\%%a.g2d %%a.crf %BFR%\%TD%\%ResDir%\%%a.crf
CD %BFR%
REM From now on we should be back to the Game's main directory
ECHO Generating the Mission files...
FOR %%a IN (1 2 3 4 5 6 7 9 10 11 12 13 14) DO GTD\xdelta patch GTD\MISS%%a.MIS.g2d MISS%%a.MIS %TD%\MISS%%a.MIS
FOR %%a IN (01 02 03 04 05 06 07 09 10 11 12 13 14) DO GTD\xdelta patch GTD\MISS%%a.OSM.g2d MISS%%a.OSM %TD%\MISS%%a.OSM
ECHO Generating the videos and a few other files...
FOR %%a IN (B09 B10 CREDITS CS06 DEATH INTRO SUCCESS) DO GTD\xdelta patch GTD\MOVIES\%%a.g2d MOVIES\%%a.avi %TD%\MOVIES\%%a.avi
FOR %%a IN (DARK.BND DARK.GAM GEN.OSM MOTIONDB.BIN) DO GTD\xdelta patch GTD\%%a.g2d %%a %TD%\%%a
REM TDP 1.33 includes an almost empty USER.CFG file ; we'll generate it without using xdelta below :
ECHO.>%TD%\USER.CFG
ECHO Extracting and generating interface fixes...
MD %TD%\intrface
MD %TD%\intrface\english
MD %TD%\intrface\french
MD %TD%\intrface\german
GTD\unzip.exe -C %ResDir%\intrface.crf optionar.bin optioncr.bin english/controls.str english/options.str -d %TD%\intrface >NUL
COPY %TD%\%ResDir%\INTRFACE.CRF %TD%\intrface\INTRFACE.CRF >NUL
GTD\xdelta patch GTD\intrface\french\controls.e2f %TD%\intrface\english\controls.str %TD%\intrface\french\controls.str
GTD\xdelta patch GTD\intrface\french\options.e2f %TD%\intrface\english\options.str %TD%\intrface\french\options.str
GTD\xdelta patch GTD\intrface\german\controls.e2g %TD%\intrface\english\controls.str %TD%\intrface\german\controls.str
GTD\xdelta patch GTD\intrface\german\options.e2g %TD%\intrface\english\options.str %TD%\intrface\german\options.str
ECHO Extracting the .DIF files...
GTD\unzip.exe GTD\DIF133.zip -d %TD% >NUL
ECHO Generating new install.cfg...
type install.cfg | GTD\grep -v -i -U "^RESNAME_BASE " | GTD\grep -i -U "[a-z]" | GTD\sed s/"^cd_path "/";cd_path "/I> %TD%\install.cfg
type install.cfg | GTD\grep -i -U "^RESNAME_BASE " | GTD\sed s/$/+.\\INTRFACE/ >> %TD%\install.cfg
REM Now we'll see if we need to generate the 1.33 EXE or not, and if we do, automatically use the right delta.
REM If the current Thief.exe is from the unofficial 'NewDark' patch, we skip that part
IF %ExeVer%==? GOTO step3_skip_exe
ECHO Generating version 1.33 Thief.exe from current version, identified as '%ExeVer%'
GTD\xdelta patch GTD\%ExeVer%.g2d Thief.exe %TD%\Thief.exe
:step3_skip_exe

:step4
ECHO.
ECHO STEP 4-a - Verifying integrity of generated files - part 1...
REM Moving down to the temporary working directory
CD %TD%
..\GTD\md5sum -c ..\GTD\TDPpart1.md5
IF ERRORLEVEL 1 SET ERR=4
IF NOT %ERR%==0 GOTO ERREUR
ECHO.
ECHO STEP 4-b - Verifying integrity of generated files - part 2...
REM Moving further down to the RES subdirectory if it exists - if not we'll still be in the Temporary directory.
REM Then BFR will save us again ^^
CD %ResDir%
%BFR%\..\GTD\md5sum -c %BFR%\..\GTD\133CRF.md5
IF ERRORLEVEL 1 SET ERR=4
IF NOT %ERR%==0 GOTO ERREUR
REM Moving back to our main Temporary Directory - if we were not inside that already
CD %BFR%
REM If we didn't generate the 1.33 Thief.exe, we skip the next part.
IF %ExeVer%==? goto step4_skip_exe
ECHO.
ECHO STEP 4-c - verifying the generated version 1.33 Thief.exe...
ECHO 99459c025fae93dcf33b2ec04c912aec *THIEF.EXE | ..\GTD\md5sum -c
IF ERRORLEVEL 1 SET ERR=4
IF NOT %ERR%==0 GOTO ERREUR
:step4_skip_exe

:step5
REM Moving back from the Temp Directory to Thief's main...
CD ..
ECHO.
ECHO Looks like everything went fine :)
ECHO.
ECHO STEP 5 (final) - Moving stuff around
REM Backing up files from Thief Gold which will either be replaced or not needed anymore
ECHO Creating backup folder '%BF%' and moving original game files there...
MD %BF%
MD %BF%\MOVIES
IF NOT %ResDir%==. MD %BF%\%ResDir%
MOVE MISS*.MIS %BF% >NUL
MOVE MISS*.OSM %BF% >NUL
FOR %%a IN (DARK.BND DARK.GAM GEN.OSM MOTIONDB.BIN CONVICT.OSM install.cfg) DO MOVE %%a %BF% >NUL
MOVE %ResDir%\*.CRF %BF%\%ResDir% >NUL
FOR %%a IN (B09 B10 B15 B16 B17 B18 CREDITS CS06 DEATH INTRO SUCCESS) DO MOVE MOVIES\%%a.avi %BF%\MOVIES >NUL
IF NOT %ExeVer%==? MOVE Thief.exe %BF% >NUL
FOR %%a IN (DROMED.CFG DROMED.EXE GOLDSKIP.EXE README.WRI THIEF_no_ddfix.EXE MISS18.MIS) DO IF EXIST %%a MOVE %%a %BF% >NUL
REM Also backing up cam.cfg and ddfix.ini to avoid possible resolution error with ddfix if user later restores from backup
COPY /y cam.cfg %BF% >NUL
IF EXIST ddfix.ini COPY /y ddfix.ini %BF% >NUL
REM Putting the new files in place
ECHO Moving generated files to their destinations...
FOR %%a IN (INTRFACE INTRFACE\ENGLISH INTRFACE\FRENCH INTRFACE\GERMAN) DO IF NOT EXIST %%a MD %%a
FOR %%a IN (INTRFACE\ENGLISH INTRFACE\FRENCH INTRFACE\GERMAN INTRFACE) DO MOVE %TD%\%%a\*.* %%a >NUL
MOVE %TD%\%ResDir%\*.CRF .\%ResDir% >NUL
MOVE %TD%\*.* . >NUL
MOVE %TD%\MOVIES\*.* .\MOVIES >NUL
ECHO Removing temporary work directories...
IF NOT %ResDir%==. RD %TD%\%ResDir%
FOR %%a IN (%TD%\MOVIES %TD%\intrface\english %TD%\intrface\french %TD%\intrface\german %TD%\intrface %TD%) DO RD %%a
ECHO.
ECHO Finished.
ECHO.
GOTO fin

:ERREUR
ECHO.
ECHO /!\ An error has occurred /!\
ECHO.
ECHO Description follows :
ECHO.
IF %ERR%==1 GOTO ERR1
IF %ERR%==2 GOTO ERR2
IF %ERR%==4 GOTO ERR4

:ERR1
ECHO Patch file(s) missing or corrupted :
ECHO.
ECHO  One or several files from this patch could not be found, or not
ECHO  properly verified.
ECHO.
ECHO  Please ensure you extracted all the files from the patch without error
ECHO  before running it, including its folder structure.
GOTO fin

:ERR2
ECHO Game file(s) missing, corrupted, or otherwise not recognized :
ECHO.
ECHO  Files from the game were not (all) found, or not all in an expected version.
ECHO.
ECHO  This patch must be extracted into the game's directory - or better yet,
ECHO  a copy of it... see the Read Me file.
ECHO  The game itself must have been installed completely, including all videos
ECHO  (copy them from the game CD if the game's setup program didn't) and in
ECHO  the 'Gold' version, english language.
GOTO fin

:ERR4
ECHO The patching procedure failed :
ECHO.
ECHO  One or several of the files that should have been generated were not,
ECHO  or failed the hash check.
ECHO.
ECHO  Consequently, the patching procedure has been interrupted.
ECHO  However this game's installation must not have been altered.
ECHO.
ECHO  You should ensure you have at the very least 390 MB free space on the drive
ECHO  and that it is not defective.
ECHO.
ECHO  Should you wish to attempt running this patch again, please remove
ECHO  the '%TD%' directory before doing so.
GOTO fin

:fin
ECHO.
ECHO Press any key to end this script.
PAUSE > NUL
